/*
NinthStar - A portable Win32 NES Emulator written in C++
Copyright (C) 2000  David de Regt

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

For a copy of the GNU General Public License, go to:
http://www.gnu.org/copyleft/gpl.html#SEC1
*/

#ifndef NES_PPU_SCANLINEH
#define NES_PPU_SCANLINEH

#include "NES_PPU.h"

class cNES_PPU_Scanline : public cNES_PPU {
	public :
		cNES_PPU_Scanline();
		~cNES_PPU_Scanline()
		{
		}

		void MemSet(unsigned short Addy, unsigned char Val)
		{
			Addy &= 0x3FFF;

			switch (MType[Addy])
			{
				case MT_CHR :
					if (Val != *(CHRPointer[(Addy & 0x3C00) >> 10] + (Addy & 0x3ff))) PatChanged = true;
					*(CHRPointer[(Addy & 0x3C00) >> 10] + (Addy & 0x3ff)) = Val;
					break;
				case MT_NTAB :
					if (Val != *(CHRPointer[(Addy & 0x3C00) >> 10] + (Addy & 0x3ff))) NTabChanged = true;
					*(CHRPointer[(Addy & 0x3C00) >> 10] + (Addy & 0x3ff)) = Val;
					break;
				case MT_PAL :
					if ((Addy & 0x3) == 0)
					{
//UnComment This when Emphasis Palette is in!
//						Val *= EmphBit;
						if (Val != Palette[0][Addy & 0x0F]) { PalChanged = true; NTabChanged = true; }
						Palette[0][Addy & 0x0F] = Val;
						Palette[1][Addy & 0x0F] = Val;
					} else {
						if (Val != Palette[(Addy & 0x0010) >> 4][Addy & 0x000F])  { PalChanged = true; NTabChanged = true; }
						Palette[(Addy & 0x0010) >> 4][Addy & 0x000F] = Val;
					}
					break;
				default :
//					MessageBox(NULL,"Invalid PPU Mem Type - Write", "0wn3d", MB_OK);
					break;
			};
		}

		unsigned char MemGet(unsigned short Addy)
		{
			switch (MType[Addy & 0x3FFF])
			{
				case MT_CHR :
					return(*(CHRPointer[(Addy & 0x3C00) >> 10] + (Addy & 0x3ff)));
				case MT_NTAB :
					return(*(CHRPointer[(Addy & 0x3C00) >> 10] + (Addy & 0x3ff)));
				case MT_PAL :
					return(*(CHRPointer[((Addy & 0x3C00) >> 10) - 4] + (Addy & 0x3ff)));
				default :
//					MessageBox(NULL,"Invalid PPU Mem Type - Read", "0wn3d", MB_OK);
					return(0);
			};
		}

		void FrameDone();
		
		void Scanline();
};

#endif